<?php
/**
 * Plugin Name: Jigoshop alfabank
 * Description: alfabank gateway for Jigoshop.
 * Author: alfabank
 * Version: 0.2
 * License: GPLv2 or later
 * Text Domain: jigoalfabank
 * Domain Path: /languages/
 */

function jigoalfabank_fetch($template, $vars = array()) {
    extract($vars);
    ob_start();
    try {
        include(WP_PLUGIN_DIR . '/' . dirname(plugin_basename(__FILE__)) . '/templates/' . $template . '.php');
        $output = ob_get_contents();
    } catch (Exception $e) {
        $output = false;
    }
    ob_end_clean();
    return $output;
}


/* сообщение о неустановленном JigoShop */
function jigoalfabank_fallback_notice() {
    echo jigoalfabank_fetch('error', array('error' => __('JigoShop alfabank Plugin depends on <a href="http://wordpress.org/extend/plugins/jigoshop/">JigoShop</a>.', 'jigoalfabank')));
}

/* загружаем наш плагин  */
add_action('plugins_loaded', 'jigoalfabank_gateway_load', 0);


function jigoalfabank_gateway_load() {
    /* если нет jigoshop выводим сообщение */
    if (!class_exists('jigoshop_payment_gateway')) {
        add_action('admin_notices', 'jigoalfabank_fallback_notice');
        return;
    }

    require_once(WP_PLUGIN_DIR . '/' . dirname(plugin_basename(__FILE__)) . '/orderv2.php');

    load_plugin_textdomain('jigoalfabank', false, dirname(plugin_basename(__FILE__)) . '/languages/');

    /* регистрируем наш платёжный метод */
    add_filter('jigoshop_payment_gateways', 'jigoalfabank_add_alfabank_gateway', 10);

    function jigoalfabank_add_alfabank_gateway($methods) {
        $methods[] = 'alfabank_Gateway';
        return $methods;
    }

    class alfabank_Gateway extends jigoshop_payment_gateway {

        public function __construct() {

            parent::__construct();

            $this->id = 'alfabank';
            $this->has_fields = false;

            $this->enabled = Jigoshop_Base::get_options()->get_option('jigoalfabank_alfabank_enabled');
            $this->title = Jigoshop_Base::get_options()->get_option('jigoalfabank_alfabank_title');
            $this->description = Jigoshop_Base::get_options()->get_option('jigoalfabank_alfabank_description');

            /* переменные */
            $this->shop_id = Jigoshop_Base::get_options()->get_option('jigoalfabank_alfabank_shop_id');
            $this->shop_url = Jigoshop_Base::get_options()->get_option('jigoalfabank_alfabank_shop_url');
            $this->shop_login = Jigoshop_Base::get_options()->get_option('jigoalfabank_alfabank_shop_login');
            $this->shop_password = Jigoshop_Base::get_options()->get_option('jigoalfabank_alfabank_shop_password');
            $this->shop_tax = Jigoshop_Base::get_options()->get_option('jigoalfabank_alfabank_shop_tax');
            
            /* действия */
            add_action('init', array(&$this, 'check_response'));
            add_action('receipt_alfabank', array(&$this, 'receipt_page'));
            add_action('jigoshop_before_order_summary_details', array(&$this, 'alfabank_showmessage'));
            add_action('alfabank_cron', array(&$this, 'alfabank_cron'));
        }

        /* настройки переменных */
        protected function get_default_options() {
            $defaults = array();

            $defaults[] = array(
                'name' => __('alfabank', 'jigoalfabank'),
                'type' => 'title',
                'desc' => __('alfabank payment system.', 'jigoalfabank')
            );

            // List each option in order of appearance with details.
            $defaults[] = array(
                'name' => __('Enable alfabank', 'jigoalfabank'),
                'desc' => '',
                'tip' => '',
                'id' => 'jigoalfabank_alfabank_enabled',
                'std' => 'yes',
                'type' => 'checkbox',
                'choices' => array(
                    'no' => __('No', 'jigoalfabank'),
                    'yes' => __('Yes', 'jigoalfabank')
                )
            );

            $defaults[] = array(
                'name' => __('Method Title', 'jigoalfabank'),
                'desc' => '',
                'tip' => __('This controls the title which the user sees during checkout.', 'jigoalfabank'),
                'id' => 'jigoalfabank_alfabank_title',
                'std' => __('alfabank', 'jigoalfabank'),
                'type' => 'text'
            );

            $defaults[] = array(
                'name' => __('Description', 'jigoalfabank'),
                'desc' => '',
                'tip' => __('This controls the description which the user sees during checkout.', 'jigoalfabank'),
                'id' => 'jigoalfabank_alfabank_description',
                'std' => __('Pay via alfabank', 'jigoalfabank'),
                'type' => 'longtext'
            );

            $defaults[] = array(
                'name' => __('alfabank Shop ID', 'jigoalfabank'),
                'desc' => '',
                'tip' => __('Please enter your Shop ID in alfabank system.', 'jigoalfabank'),
                'id' => 'jigoalfabank_alfabank_shop_id',
                'std' => '',
                'type' => 'text'
            );

            $defaults[] = array(
                'name' => __('alfabank Gateway Address', 'jigoalfabank'),
                'desc' => '',
                'tip' => __('Please enter alfabank gateway URL-address.', 'jigoalfabank'),
                'id' => 'jigoalfabank_alfabank_shop_url',
                'std' => '',
                'type' => 'text'
            );

            $defaults[] = array(
                'name' => __('alfabank login', 'jigoalfabank'),
                'desc' => '',
                'tip' => __('Please enter your login in alfabank system.', 'jigoalfabank'),
                'id' => 'jigoalfabank_alfabank_shop_login',
                'std' => '',
                'type' => 'text'
            );

            $defaults[] = array(
                'name' => __('alfabank password', 'jigoalfabank'),
                'desc' => '',
                'tip' => __('Please enter your password in alfabank system.', 'jigoalfabank'),
                'id' => 'jigoalfabank_alfabank_shop_password',
                'std' => '',
                'type' => 'text'
            );

            $defaults[] = array(
                'name' => __('alfabank tax percentage', 'jigoalfabank'),
                'desc' => '',
                'tip' => __('Please enter tax percentage', 'jigoalfabank'),
                'id' => 'jigoalfabank_alfabank_shop_tax',
                'std' => '0',
                'type' => 'text'
            );


            return $defaults;
        }

        /* подготавливаем новое soap-соединение */
        function newSoap() {
            return new orderv2(null, array(
                'location' => $this->shop_url,
                'uri'      => 'http://www.sirena-travel.ru',
                'login'    => $this->shop_login,
                'password' => $this->shop_password,
                'trace'    => 0,
                'features' => SOAP_SINGLE_ELEMENT_ARRAYS,
                'connection_timeout' => 12));
        }

        /* подготавливаем данные заказа для отправки */
        function prepareOrder($orderId, $orderItems, $shipping, $orderSum, $userId, $userName, $userEmail, $userPhone, $okUrl, $faultUrl) {
            $request = new register();
            $order = new OrderID(); 
            $order->shop_id = $this->shop_id;
            $order->number = $orderId;
            $cost = new Amount();
            $cost->amount = $orderSum;
            $cost->currency = 'RUB';
            $customer = new CustomerInfo();
            if ($userId != 0) {
                $customer->id = $userId;
            }
            $customer->name = $userName;
            $customer->email = $userEmail;
            $customer->phone = $userPhone;
            $description = new OrderInfo();
            $description->paytype = 'card';
            $language = new PostEntry();
            $language->name = 'Language';
            $language->value = 'ru';
            $returnUrlOk = new PostEntry();
            $returnUrlOk->name = 'ReturnURLOk';
            $returnUrlOk->value = $okUrl;
            $returnUrlFault = new PostEntry();
            $returnUrlFault->name = 'ReturnURLFault';
            $returnUrlFault->value = $faultUrl;
            $cardtype = new PostEntry();
            $cardtype->name = 'ChoosenCardType';
            $cardtype->value = 'VI';
            $request->order = $order;
            $request->cost = $cost;
            $request->customer = $customer;

            $arItems = array();


        

            foreach ($orderItems as $item) {
                $item_amount = number_format(apply_filters('jigoshop_paypal_adjust_item_price', $item['cost'], $item, 10, 2), $this->decimals);
                $itemCost = new Amount();
                $itemCost->amount = $item_amount * $item['qty']; 
                $itemCost->currency = 'RUB';

                if ($this->shop_tax == 'no' || (int)$this->shop_tax == 0){
                    $itemTaxes = array(
                            new SoapVar("<tax type='vat'><percentage>".$this->shop_tax."</percentage><amount><currency>RUB</currency><amount>0</amount></amount></tax>", XSD_ANYXML),
                        );
                } else {
                    $price = $item_amount / (1 + (int)$this->shop_tax / 100);
                    $taxAmount = $item_amount - $price;
                    $itemTaxes = array(
                            new SoapVar("<tax type='vat'><percentage>".$this->shop_tax."</percentage><amount><currency>RUB</currency><amount>".round($taxAmount, 2) * $item['qty']."</amount></amount></tax>", XSD_ANYXML),
                        );
                }

                $orderItem = new OrderItem();
                $orderItem->number = $orderId;
                $orderItem->amount = $itemCost;
                $orderItem->typename = 'good';
                $orderItem->quantity = $item['qty'];
                $orderItem->name = $item['name'];
                $orderItem->taxes = new SoapVar($itemTaxes, SOAP_ENC_OBJECT);
                
                $arItems[] = new SoapVar($orderItem, SOAP_ENC_OBJECT, null, null, 'OrderItem');   

            }

            // Shipping Cost
            if (jigoshop_shipping::is_enabled() && $shipping > 0) {
                $item_amount = number_format((float)$shipping, $this->decimals);
                $itemCost = new Amount();
                $itemCost->amount = $item_amount; 
                $itemCost->currency = 'RUB';

                if ($this->shop_tax == no || (int)$this->shop_tax == 0){
                    $itemTaxes = array(
                            new SoapVar("<tax type='vat'><percentage>".$this->shop_tax."</percentage><amount><currency>RUB</currency><amount>0</amount></amount></tax>", XSD_ANYXML),
                        );
                } else {
                    $price = $item_amount / (1 + (int)$this->shop_tax / 100);
                    $taxAmount = $item_amount - $price;
                    $itemTaxes = array(
                            new SoapVar("<tax type='vat'><percentage>".$this->shop_tax."</percentage><amount><currency>RUB</currency><amount>".round($taxAmount, 2)."</amount></amount></tax>", XSD_ANYXML),
                        );
                }
                $item = new OrderItem();
                $item->number = $orderId;
                $item->amount = $itemCost;
                $item->typename = 'service';
                $item->quantity = 1;
                $item->name = 'Shipping';
                $item->taxes = new SoapVar($itemTaxes, SOAP_ENC_OBJECT);
                $arItems[] = new SoapVar($item, SOAP_ENC_OBJECT, null, null, 'OrderItem');
            }
            $description->sales = new SoapVar($arItems, SOAP_ENC_OBJECT);




            $request->description = $description;

            $postdata = new SoapVar(array(
                                            new SoapVar($language, SOAP_ENC_OBJECT, null, null, 'PostEntry'),
                                            new SoapVar($cardtype, SOAP_ENC_OBJECT, null, null, 'PostEntry'),
                                            new SoapVar($returnUrlOk, SOAP_ENC_OBJECT, null, null, 'PostEntry'),
                                            new SoapVar($returnUrlFault, SOAP_ENC_OBJECT, null, null, 'PostEntry'),
                                    ), SOAP_ENC_OBJECT);

            $request->postdata = $postdata;

            return $request;
        }

        /* подготавливаем структуру для запроса статуса заказа */
        public function getOrderStatus($orderId) {
            $status = new get_status();
            $order = new OrderID();
            $order->shop_id = $this->shop_id;
            $order->number = $orderId;
            $status->order = $order;
            return $status;
        }

        /* выводим описание нашей платёжной системы */
        function payment_fields() {
            if ($this->description) {
                echo wpautop(wptexturize($this->description));
            }
        }

        function process_payment($order_id) {
            $order = new jigoshop_order($order_id);

            return array(
                'result' => 'success',
                'redirect' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(jigoshop_get_page_id('pay'))))
            );
        }


        /* выводим кнопку и регистрируем заказ */
        function receipt_page($order_id) {
            $order = new jigoshop_order($order_id);

            $soapClient = $this->newSoap();
            $request = $this->prepareOrder(
                $order->id,
                $order->items,
                $order->order_shipping,
                $this->fix_money_format($order->order_total),
                $order->user_id,
                $order->billing_last_name . ' ' . $order->billing_first_name,
                $order->billing_email,
                $order->billing_phone,
                add_query_arg('alfabank_status', 'ok', add_query_arg('key', $order->order_key, add_query_arg('order', $order->id, trailingslashit(get_bloginfo('url'))))),
                add_query_arg('alfabank_status', 'fault', add_query_arg('key', $order->order_key, add_query_arg('order', $order->id, trailingslashit(get_bloginfo('url')))))
            );
            try {
                $info = $soapClient->register($request);
                // всё ок, зарегистрировали, перенаправляем на платёжную систему
                echo jigoalfabank_fetch('register', array(
                    'form_url' => $info->redirect_url . '?session=' . $info->session,
                    'cancel_url' => esc_url($order->get_cancel_order_url())
                ));
            } catch (SoapFault $fault) {
                echo jigoalfabank_fetch('register.error', array(
                    'url' => add_query_arg('order', $order->id, add_query_arg('key', $order->order_key, get_permalink(jigoshop_get_page_id('pay'))))
                ));
            } catch (Exception $e) {
                echo jigoalfabank_fetch('error', array('error' => $e->__toString()));
            }
            foreach ($order->items as $item) {
                    foreach ($item as $val=>$key) {
                        echo $val.'----->'.$key.'<br>';
                    } 
                } 
        }


        function fix_money_format($value) {
            $decimal_sep = Jigoshop_Base::get_options()->get_option('jigoshop_price_decimal_sep');
            $thousand_sep = Jigoshop_Base::get_options()->get_option('jigoshop_price_thousand_sep');

            if ($thousand_sep == '.' && $decimal_sep == ',') {
                $value = str_replace('.', '', $value);
                $value = str_replace(',', '.', $value);
            }

            $value = number_format($value, 2, '.', '');

            return $value;
        }

        function alfabank_showmessage() {
            if (isset($_GET['error'])) {
                switch ($_GET['error']) {
                    case 'fault':
                        echo jigoalfabank_fetch('error', array('error' => __('Payment has failed.', 'jigoalfabank')));
                    break;

                    case 'invalid':
                        echo jigoalfabank_fetch('error', array('error' => __("Order with this number has not registered. Please, contact shop's support service.", 'jigoalfabank')));
                    break;

                    case 'notregistered':
                        echo jigoalfabank_fetch('error', array('error' => __('Wrong order number or key.', 'jigoalfabank')));
                    break;
                }
            }
        }


        public function check_response() {
            if (is_admin()) {
                return;
            }

            if (isset($_GET['alfabank_status'])) {
                $order = new jigoshop_order((int) $_GET['order']);
                if ($order->order_key === $_GET['key']) {
                    if ($_GET['alfabank_status'] == 'ok') {
                        $soapClient = $this->newSoap();
                        $objStatus = $this->getOrderStatus($order->id);
                        // получаем статуса заказа от платёжной системы
                        $info = $soapClient->get_status($objStatus);
                        try {
                            switch ($info->status) {
                                case 'acknowledged':
                                case 'not_acknowledged':
                                case 'authorized':
                                    $order->add_order_note(__('Payment completed.', 'jigoalfabank'));
                                    $order->payment_complete();
                                    wp_redirect(get_permalink(jigoshop_get_page_id('thanks')));
                                    die();
                                break;

                                case 'canceled':
                                case 'not_authorized':
                                    $order->update_status('cancelled', __('Payment canceled by alfabank.', 'jigoalfabank'));
                                break;
                    
                                default:
                                break;
                            }
                        } catch (SoapFault $fault) {
                            if ($fault->faultstring === 'INVALID_ORDER') {
                                $order->update_status('cancelled', __('Payment canceled by alfabank.', 'jigoalfabank'));
                            }
                        } catch (Exception $e) {
                            echo jigoalfabank_fetch('error', array('error' => $e->__toString()));
                        }
                    }
                    if ($_GET['alfabank_status'] == 'fault') {
                        wp_redirect(add_query_arg('error', 'fault', add_query_arg('order', $order->id, get_permalink(jigoshop_get_page_id('view_order')))));
                        die();
                    }
                } else {
                    wp_redirect(add_query_arg('error', 'wrong_id', add_query_arg('order', $_GET['order'], get_permalink(jigoshop_get_page_id('view_order')))));
                    die();
                }
                wp_redirect(add_query_arg('order', $_GET['order'], get_permalink(jigoshop_get_page_id('view_order'))));
                die();
            }
        }

        function alfabank_cron() {
            $args = array(
                'numberposts' => -1,
                'meta_key' => '',
                'meta_value' => '',
                'post_type' => 'shop_order',
                'post_status' => 'publish',
                'tax_query' => array(
                    array(
                        'taxonomy' => 'shop_order_status',
                        'terms' => array('pending'),
                        'field' => 'slug',
                        'operator' => 'IN'
                    )
                ),
                'meta_query' => array(
                    array(
                        'key' => 'order_data',
                        'value' => $this->id,
                        'compare' => 'LIKE'
                    )
                )
            );

            $results = get_posts($args);

            if ($results) {
                $soapClient = $this->newSoap();

                foreach ($results as $result) {
                    $order = new jigoshop_order();
                    $order->populate($result);

                    if ($order->payment_method == $this->id) {
                        $objStatus = $this->getOrderStatus($order->id);
                        try {
                            $info = $soapClient->get_status($objStatus);

                            switch ($info->status) {
                                case 'acknowledged':
                                case 'not_acknowledged':
                                case 'authorized':
                                    $order->add_order_note(__('Payment completed.', 'jigoalfabank'));
                                    $order->payment_complete();
                                break;

                                case 'canceled':
                                case 'not_authorized':
                                    // оплата заказа не удалась
                                    $order->update_status('cancelled', __('Payment canceled by alfabank.', 'jigoalfabank'));
                                break;
                            }
                            echo 'Статус заказа № ', $order->id, ': ', $info->status, "\r\n";

                        } catch (SoapFault $fault) {
                            echo 'Ошибка при проверке статуса заказа № ', $order->id, ': ', $fault->faultcode , '-' , $fault->faultstring, "\r\n";
                            if ($fault->faultstring === 'INVALID_ORDER') {
                                // заказ с таким номером не зарегистрирован
                                $order->update_status('cancelled', __('Payment canceled by alfabank.', 'jigoalfabank'));
                            }
                        }
                    }

                    unset($order);
                }
              
            }
        }

    }
}